-- Title     :  Render Region - 3DS MAX scripted RenderEffect. Some of renderers such as Arnold or Mental Ray after render finished clear outside area, so you lose previous rendered  parts of images in VFB. Render Region restores cleared area of last rendered image.
--              
--
-- Copyright :  2007-2017 veda3d.com, All rights reserved
-- Author    : Royal Ghost
-- Version   : 1.2.0
--		- Added Paste image  from Windows clipboard to internal RegionRender buffer
--		- Added View and thumbnail of internal RR buffer
--		- Added Clear of  internal RR buffer
-- Version   : 1.1.0
--	       - Added 3dsmax 2017 support with HighDPI fix.
-- Version   : 1.0.1
--	       - Added 3dsmax 2008 support.
-- Homepage  : http://www.veda3d.com
---------------------------------------------------------------------------------------------
--             MODIFY THIS AT YOUR OWN RISK

plugin renderEffect RegionRender
name:"Region Render"
classID:#(0x57d5a641, 0xc6a4086)
( 
local lrbmp, functional 
local lrbmp_thumbSize = [200,113]
local lrbmp_thumb = (bitmap lrbmp_thumbSize.x lrbmp_thumbSize.y color:black)
local bgColor = (((colorMan.getColor #background) as point4) as color) *0.4545

fn funct =
(
maxv=maxVersion()
	
	if avguard_dlx_ver != undefined or maxv[1] >= 10000 then 
	(
	f = true
	)
	else
    (
	f = false
	messagebox "avg_dlx.dlx not installed. Render Effect not functional!" title:"Render Effect"
	)
return f
)
fn region_fix region =
(
if region.y < 0 then region.y = 0
if region.x < 0 then region.x = 0
return region
)
fn generate_thumbnail =
(
local tmpbmp
			if lrbmp.width > lrbmp.height then
			(
				ratio = lrbmp_thumbSize.x/lrbmp.width
				tmpbmp = bitmap (lrbmp.width*ratio) (lrbmp.height*ratio) 
				copy lrbmp tmpbmp
				lrbmp_thumb	= bitmap lrbmp_thumbSize.x lrbmp_thumbSize.y color:bgColor
				try pasteBitmap tmpbmp lrbmp_thumb [0,0] [0,((lrbmp_thumbSize.y/2)-(tmpbmp.height/2))] catch()
			)
			else
			(
				ratio = lrbmp_thumbSize.y /lrbmp.height
				tmpbmp = bitmap (lrbmp.width*ratio) (lrbmp.height*ratio)
				copy lrbmp tmpbmp
				lrbmp_thumb	= bitmap lrbmp_thumbSize.x lrbmp_thumbSize.y color:bgColor
				try pasteBitmap tmpbmp lrbmp_thumb [0,0] [((lrbmp_thumbSize.x/2)-(tmpbmp.width/2)),0] catch()			
			)	
)

parameters main rollout:params
(
)
rollout params "Render Region v1.2.0 Parameters"
(
button btn_clr "Clear" width:90 height:24 align:#left tooltip:"Clear internal RR buffer."
button btn_pasteBuff "Paste" width:90 height:24 align:#left tooltip:"Paste image from image clipboard to RR internal buffer. In VFB image will be shown after render."
button btn_viewBuff "View" width:90 height:24 align:#left tooltip:"Display image from internal RR buffer."
bitmap bm_thumbnail bitmap:lrbmp_thumb align:#right offset:[0,-90]
label lbl_author "by Royal Ghost" align:#right 
hyperLink url_v3d "veda3d.com" address:"http://veda3d.com" align:#right

	
	on btn_clr pressed  do
	(
	b = bitmap renderWidth renderHeight color:black
	if lrbmp != undefined then copy b lrbmp
		
	copy lrbmp lrbmp_thumb		
	bm_thumbnail.bitmap = lrbmp_thumb
	)
	
	on btn_pasteBuff pressed  do
	(
	clpbmp = getclipboardBitmap()
	if lrbmp == undefined and clpbmp != undefined then lrbmp = bitmap clpbmp.width clpbmp.height gamma:1.0
	try pasteBitmap clpbmp lrbmp [0,0] [0,0] catch()-- convert gamma
	
	generate_thumbnail()
	bm_thumbnail.bitmap = lrbmp_thumb

	)
	on btn_viewBuff pressed  do
	(
	if lrbmp != undefined then display lrbmp

	)
on params open do
(
	if lrbmp != undefined then 
	(
	generate_thumbnail()		
	bm_thumbnail.bitmap = lrbmp_thumb
	)
	else 
	(
	bm_thumbnail.bitmap = lrbmp_thumb
	)
)
)
on create do 
(
functional = funct()
)
on apply bmp do
(
functional = funct()
if functional==true then 
		(
		if getrendertype() == #region and lrbmp != undefined then
		(
		vpRegion = viewport.getRegionRect viewport.activeViewport
		vpRegion = region_fix vpRegion
		try UIScaleFactor = GetUIScaleFactor() catch (UIScaleFactor = 1.0)
		vpRegion = box2 (formattedPrint (vpRegion.x*UIScaleFactor) format:".0f" as integer) \
				     	(formattedPrint (vpRegion.y*UIScaleFactor) format:".0f" as integer) \
						(formattedPrint (vpRegion.w*UIScaleFactor-1) format:".0f" as integer) \
						(formattedPrint (vpRegion.h*UIScaleFactor-1) format:".0f" as integer)
			if lrbmp != undefined then
			(
				if [lrbmp.width,lrbmp.height] == [renderWidth,renderHeight] then
				(
				pasteBitmap bmp lrbmp vpRegion [vpRegion.x,vpRegion.y] type:#paste
				copy lrbmp bmp
				)
			)
		)
		lrbmp = getLastRenderedImage()

		generate_thumbnail()
		params.bm_thumbnail.bitmap = lrbmp_thumb
		)
gc light:true
)
)
